<?php

class Admin extends Model {

    public static function estAdmin($slug) {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_admin WHERE identite = :slug"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute(array(":slug" => $slug));
        return ($req->rowcount() != 0);
    }

    public static function getListeJeuNonValide() {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_jeu WHERE NOT est_valide ORDER BY nom"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute();
        $res = array();
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
            $res[] = $ligne;
        }
        return $res;
    }

    public static function acceptJeu($slug) {
        $db = Database::getInstance();
        $sql = "UPDATE R_jeu SET est_valide = 1 WHERE slug = :slug";
        $req = $db->prepare($sql);
        return $req->execute(array(':slug' => $slug));
    }

    public static function supprJeu($slug) {
        $db = Database::getInstance();
        $sql= "SELECT * FROM R_partie WHERE jeu = :slug";
        $req = $db->prepare($sql);
        $req->execute(array(':slug' => $slug));
        $sql= "DELETE FROM R_joueur_partie WHERE partie = :partie";
        $req = $db->prepare($sql);
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) {
            $req->execute(array(':partie' => $ligne['id']));
        }
        $sql= "DELETE FROM R_partie WHERE jeu = :slug";
        $req = $db->prepare($sql);
        $req->execute(array(':slug' => $slug));
        $sql= "DELETE FROM R_joueur_jeu WHERE jeu = :slug";
        $req = $db->prepare($sql);
        $req->execute(array(':slug' => $slug));
        $sql = "DELETE FROM R_jeu WHERE slug = :slug";
        $req = $db->prepare($sql);
        $req->execute(array(':slug' => $slug));
    }

    public static function getListeJeuValide() {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_jeu WHERE est_valide ORDER BY nom"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute();
        $res = array();
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
            $res[] = $ligne;
        }
        return $res;
    }
    
    public static function unacceptJeu($slug) {
        $db = Database::getInstance();
        $sql = "UPDATE R_jeu SET est_valide = 0 WHERE slug = :slug";
        $req = $db->prepare($sql);
        return $req->execute(array(':slug' => $slug));
    }
    
    public static function setModifJeu($slug, $data) {
        $db = Database::getInstance();
        $sql = "UPDATE R_jeu SET description = :description, max_joueurs = :max_joueurs,
                min_joueurs = :min_joueurs, max_equipes = :max_equipes, min_equipes = :min_equipes,
                 type = :type, type_scores = :type_scores WHERE slug = :slug";
        $req = $db->prepare($sql);
        return $req->execute(array(':slug' => $slug,
                                    ':description' => $data['description'] ,
                                    ':type' => $data['type'],
                                    ':max_joueurs' => $data['max_joueurs'],
                                    ':min_joueurs' => $data['min_joueurs'],
                                    ':max_equipes' => $data['max_equipes'],
                                    ':min_equipes' => $data['min_equipes'],
                                    ':type_scores' => $data['type_scores']));
    }

    public static function supprJoueur($slug) {
        $db = Database::getInstance();
        $sql= "DELETE FROM R_joueur_partie WHERE joueur = :slug";
        $req = $db->prepare($sql);
        $req->execute(array(':slug' => $slug));
        $sql= "DELETE FROM R_joueur_jeu WHERE joueur = :slug";
        $req = $db->prepare($sql);
        $req->execute(array(':slug' => $slug));
        $sql = "DELETE FROM R_joueur WHERE slug = :slug";
        $req = $db->prepare($sql);
        $req->execute(array(':slug' => $slug));
    }

    public static function getListPartie() {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_partie JOIN R_jeu ON R_partie.jeu = R_jeu.slug ORDER BY date"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute();
        $res = array();
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
            $res[] = $ligne;
        }
        return $res;
    }

    public static function getEquipes($id) {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_joueur_partie WHERE partie = :id ORDER BY classement"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute(array(':id' => $id));
        $res = array();
        $i = -1;
        $classement_old = 0;
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) {
            if($classement_old != $ligne['classement']) {
                $i = $i +1;
                $classement_old = $ligne['classement'];
                $res[$i] = array();
            }
            $res[$i][] = Profil::getFromSlug($ligne['joueur']);
        }
        return $res;
    }

    public static function getListeJoueur($id) {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_joueur_partie WHERE partie = :id ORDER BY classement"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute(array(':id' => $id));
        $res = array();
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
            $res[] = Profil::getFromSlug($ligne['joueur']);
        }
        return $res;
    }

    public static function getListeScore($id) {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_joueur_partie WHERE partie = :id ORDER BY classement"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute(array(':id' => $id));
        $res = array();
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
            $res[$ligne['joueur']] = $ligne['score'];
        }
        return $res;
    }

    public static function getListeScorePartie($id) {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_joueur_partie WHERE partie = :id ORDER BY classement"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute(array(':id' => $id));
        $res = array();
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
            $res[$ligne['joueur']] = $ligne['score_partie'];
        }
        return $res;
    }

    public static function getListeScorePartie2($id) {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_joueur_partie WHERE partie = :id ORDER BY classement"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute(array(':id' => $id));
        $res = array();
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
            $res[] = $ligne['score_partie'];
        }
        return $res;
    }

    public static function getListeClassement($id) {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_joueur_partie WHERE partie = :id ORDER BY classement"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute(array(':id' => $id));
        $res = array();
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
            $res[$ligne['joueur']] = $ligne['classement'];
        }
        return $res;
    }

    public static function getListeClassement2($id) {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_joueur_partie WHERE partie = :id ORDER BY classement"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute(array(':id' => $id));
        $res = array();
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
            $res[] = $ligne['classement'];
        }
        return $res;
    }

    public static function ModifPartie($id, $joueurs, $scores, $jeu)
    {
        $db = Database::getInstance();
        $sql5 = "UPDATE R_joueur_jeu SET score = :new_score WHERE joueur = :id_joueur AND jeu = :id_jeu;";
        $stmt5 = $db->prepare($sql5);

        $partieModif = Partie::getFromId($id);
        // recherche des joueurs ayant fais une partie apres celle-ci (celle-ci incluse)
        $sql1 = "SELECT * FROM R_joueur_partie JOIN R_partie ON R_joueur_partie.partie = R_partie.id WHERE date >= date(:date) AND jeu =:jeu ORDER BY date";
        $stmt1 = $db->prepare($sql1);
        $stmt1->execute(array(':date' => $partieModif->date, ":jeu" => $partieModif->jeu));
        // pour tout les joueurs remettre les scrores comme-ci aucune partie après celle-ci a été jouée
        $joueurDejaTraite = array();
        while ($joueur = $stmt1->fetch(PDO::FETCH_ASSOC)) {
            if (!in_array($joueur['joueur'], $joueurDejaTraite)) {
                $joueurDejaTraite[] = $joueur['joueur'];
                $stmt5->execute(array(':new_score' => $joueur['score'],
                ':id_joueur' => $joueur['joueur'],
                    ':id_jeu' => $joueur['jeu']));
            }
        }
        // applique la partie
        if (($resp = self::appliquePartie($id, $joueurs, $scores, $jeu)) > 0){
            print $resp;
            die();
        }
        // pour chaque partie apres celle modifier on la réapplique avec les nouveaux scores
        $sql2 = "SELECT * FROM R_partie WHERE date > date(:date) AND jeu =:jeu ORDER BY date";
        $stmt2 = $db->prepare($sql2);
        $stmt2->execute(array(':date' => $partieModif->date, ":jeu" => $partieModif->jeu));
        while ($ligne = $stmt2->fetch(PDO::FETCH_ASSOC)) {
            $joueurs = $jeu->type == 'equipes' ? Admin::getEquipes($ligne['id']) : Admin::getListeJoueur($ligne['id']);
            $scores = $jeu->type_scores == 'scores' ? Admin::getListeScorePartie2($ligne['id']) : Admin::getListeClassement2($ligne['id']);
            self::appliquePartie($ligne['id'],$joueurs, $scores, $jeu);
        }

    }


    static function appliquePartie($id_partie, $joueurs, $scores, $jeu) {
        $db = Database::getInstance();
        $sql3 = "SELECT score FROM R_joueur_jeu WHERE joueur = :id_joueur AND jeu = :id_jeu;";
        $sql4 = "UPDATE R_joueur_partie SET classement = :classement, score = :score, score_partie = :score_partie
             WHERE joueur = :joueur AND partie = :partie;";
        $sql5 = "UPDATE R_joueur_jeu SET score = :new_score WHERE joueur = :id_joueur AND jeu = :id_jeu;";
        $sqlN = "INSERT INTO R_joueur_jeu (joueur, jeu, score) VALUES (:id_joueur, :id_jeu, :score);";
        $stmt3 = $db->prepare($sql3);
        $stmt4 = $db->prepare($sql4);
        $stmt5 = $db->prepare($sql5);

        $m = count($scores);

        $old_score = array_fill(0, $m, 0);
        $old_score_e = array();
        foreach ($joueurs as $k => $joueur) {
            // version avec équipes
            if ($jeu->type == 'equipes') {
                $old_score_e[$k] = array();
                foreach ($joueur as $kj => $j) {
                    if ($stmt3->execute(array(":id_joueur" => $j->slug, ":id_jeu" => $jeu->slug))) {
                        if ($row = $stmt3->fetch()) {
                            // Le joueur a déja joué à ce jeu
                            $old_score_e[$k][$kj] = $row[0];
                        } else {
                            // ou le joueur n'a jamais joué à ce jeu, on je rajoute à la relation.
                            $old_score_e[$k][$kj] = 1000;
                            $stmtN = $db->prepare($sqlN);
                            if ($stmtN->execute(array(":id_joueur" => $j->slug, ":id_jeu" => $jeu->slug, ":score" => 1000))) {

                            } else
                                return 2;
                        }
                    }
                }
                $old_score[$k] = array_sum($old_score_e[$k]) / count($old_score_e[$k]);
            } else {
                // récupération du score actuel de chaque joueur
                if ($stmt3->execute(array(":id_joueur" => $joueur->slug, ":id_jeu" => $jeu->slug))) {
                    if ($row = $stmt3->fetch()) {
                        // Le joueur a déja joué à ce jeu
                        $old_score[$k] = $row[0];
                    } else {
                        // ou le joueur n'a jamais joué à ce jeu, on je rajoute à la relation.
                        $old_score[$k] = 1000;
                        $stmtN = $db->prepare($sqlN);
                        if ($stmtN->execute(array(":id_joueur" => $joueur->slug, ":id_jeu" => $jeu->slug, ":score" => 1000))) {

                        } else
                            return 2;
                    }

                }
            }
        }
        // Calcul de la nouvelle différence de scores et du classement de chacun dans la partie
        $diff_scores = ELO_scores($old_score, $scores, 32);
        $classement = Partie::calcul_classement($diff_scores);
        foreach ($joueurs as $k => $joueur) {
            if ($jeu->type_scores == 'classement'){
                $score = null;
            }
            else{
                $score = $scores[$k];
            }
            if ($jeu->type == 'equipes') {
                foreach ($joueur as $kj => $j) {
                    // Insertion des infos de la partie (avec le score avant la partie et le classement interne à la partie
                    // et le score de la partie) pour chaque joueur par équipe
                    if ($stmt4->execute(array(":partie" => $id_partie,
                        ":joueur" => $j->slug,
                        ":classement" => $classement[$k], ":score" => $old_score_e[$k][$kj], ":score_partie" => $score))
                    ) {
                        $newS = $old_score_e[$k][$kj] + $diff_scores[$k];
                        // On met à jour le score du joueur actuel dans sa relation avec le jeu
                        if ($stmt5->execute(array(":new_score" => $newS,
                            ":id_jeu" => $jeu->slug, ":id_joueur" => $j->slug))
                        ) {
                        } else
                            return 4;
                    } else
                        return 3;
                }
            } else {
                // Insertion des infos de la partie (avec le score avant la partie et le classement interne à la partie
                //  et le score de la partie) pour chaque joueur

                if ($stmt4->execute(array(":partie" => $id_partie,
                    ":joueur" => $joueur->slug,
                    ":classement" => $classement[$k], ":score" => $old_score[$k], ":score_partie" => $score))
                ) {
                    $newS = $old_score[$k] + $diff_scores[$k];
                    // On met à jour le score du joueur actuel dans sa relation avec le jeu
                    if ($stmt5->execute(array(":new_score" => $newS,
                        ":id_jeu" => $jeu->slug, ":id_joueur" => $joueur->slug))
                    ) {
                    } else
                        return 4;
                } else
                    return 3;
            }
        }
        // met à jour le classement de tous les joueurs
        return Partie::updateClassement($jeu);
    }

}